//  BMPx - The Dumb Music Player
//  Copyright (C) 2005 BMPx development team.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  --
//
//  The BMPx project hereby grants permission for non-GPL compatible GStreamer
//  plugins to be used and distributed together with GStreamer and BMPx. This
//  permission is above and beyond the permissions granted by the GPL license
//  BMPx is covered by.

#ifndef BMP_UI_PART_ALBUMS_HH 
#define BMP_UI_PART_ALBUMS_HH

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <glibmm.h>
#include <glibmm/markup.h>
#include <gtkmm.h>
#include <libglademm.h>

#include <mcs/mcs.h>

#ifdef HAVE_HAL
#  include "hal.hh"
#endif

#include "amazon.hh"
#include "xds.hh"
#include "library.hh"
#include "playback_source.hh"
#include "vfs.hh"

namespace Bmp
{
  class MainUI;
  class AlbumView : public Gtk::TreeView
  {
    public:

      enum ViewSize { NORMAL, SMALL };

    private:

      // Data Store Handling
      int
      default_sort_func (Gtk::TreeModel::iterator const& iter_a,
                         Gtk::TreeModel::iterator const& iter_b);
      bool
      row_is_visible (Gtk::TreeModel::iterator const& iter);
     
      void
      match_album_from_iter  (Gtk::TreeModel::iterator const& m_iter, Library::MatchAlbum &album);

      Library::SimpleAlbum
      simple_album_from_iter (Gtk::TreeModel::iterator const& m_iter);

      Glib::RefPtr<Gtk::ListStore>        m_store_base;
      Glib::RefPtr<Gtk::TreeModelFilter>  m_store_filter;
      Glib::RefPtr<Gdk::Pixbuf>           m_disc_normal;
      Glib::RefPtr<Gdk::Pixbuf>           m_disc_small;
      Glib::ustring                       m_sort;

      // Viewsize
      ViewSize m_viewsize; 
      bool m_show_only_new;

      // Selection Handling
      typedef std::pair < unsigned int, Gtk::TreeModel::RowReference > SelectionPair; 
      typedef std::list < SelectionPair > SelectionList; 
      bool record_selection_change (Glib::RefPtr<Gtk::TreeModel> const&, Gtk::TreeModel::Path const&, bool);
      bool m_selection_list_modified;
      bool m_selection_change_block;
      SelectionList m_selection_list_albums, m_selection_list_albums_saved;

      // Misc
      Gtk::CellRendererText *cell_album_text;

    public:

      void save_context  ()
      {
        if (m_selection_list_modified)
        {
          m_selection_list_albums_saved = m_selection_list_albums;
          m_selection_list_modified = false;
        }
      }

      void restore_context ()
      {
        m_sort.clear();
        m_store_filter->refilter ();
        get_selection()->unselect_all ();
        m_selection_list_albums.clear (); 
        for (SelectionList::const_iterator i = m_selection_list_albums_saved.begin (),
                                                   e = m_selection_list_albums_saved.end(); i != e; ++i)
        {
          get_selection()->select (i->second.get_path());
        }
      }

      enum AlbumFlags
      {
        A_NONE            = 0,
        A_HAS_COVER       = 1 << 0,
        A_HAS_SORTNAME    = 1 << 1
      };

      class Albums : public Gtk::TreeModel::ColumnRecord
      {
        public:

          Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> >  cover;
          Gtk::TreeModelColumn<Glib::ustring> title;
          Gtk::TreeModelColumn<AlbumFlags>    flags;
          Gtk::TreeModelColumn<unsigned int>  rowuid;
          Gtk::TreeModelColumn<Glib::ustring> artist;
          Gtk::TreeModelColumn<Glib::ustring> album;
          Gtk::TreeModelColumn<oustring>      release_id;
          Gtk::TreeModelColumn<oustring>      artist_id;
          Gtk::TreeModelColumn<oustring>      asin;

          Albums ()
          {
            add (cover);
            add (title);
            add (flags);
            add (rowuid);
            add (artist);
            add (album);
            add (release_id);
            add (artist_id);
            add (asin);
          }
      };
      Albums albums; 

      bool find_matching_album (Library::MatchAlbum&      album,
                                Gtk::TreeModel::iterator& m_iter);
      bool find_matching_album (Library::MatchAlbum &ai);

      void set_sorting    (Glib::ustring const& sort_text);
      void set_viewsize   (ViewSize viewsize);

#ifdef HAVE_HAL
      void albums_modify (Bmp::Library::PUDIVRPV const& pairs);
#else
      void albums_modify (Bmp::VFS::UriList const& locations);
#endif

      bool approve_selected ();
      bool relocate_album   ();
      void remove_albums    (bool force);
      bool update           (bool new_items);

      void get_album_list   (Library::AlbumList &list, Glib::ustring& label_text, bool& multiple);

      AlbumView (BaseObjectType                        *cobject,
                 const Glib::RefPtr<Gnome::Glade::Xml> &xml);
      ~AlbumView () {}
  };

  class TrackView : public Gtk::TreeView
  {
    private:

      typedef std::map <Glib::ustring, Gtk::TreeModel::RowReference> RowMapTracks;

      void tracks_cell_data_func (Gtk::CellRenderer* cell, Gtk::TreeModel::iterator const& m_iter);
      bool tracks_select_func (Glib::RefPtr<Gtk::TreeModel> const& model, Gtk::TreeModel::Path const& path, bool selected);

      RowMapTracks m_row_map_tracks;
      void notify_remove (Glib::ustring& uri);

      Gtk::CellRenderer *cell_playing;
      Gtk::CellRenderer *cell_rating;

      Glib::RefPtr<Gdk::Pixbuf>     rating[6];
      Glib::RefPtr<Gtk::ListStore>  store_tracks;
      Glib::RefPtr<Gdk::Pixbuf>     playing;

      int           m_sort_column_saved;
      Gtk::SortType m_sort_order_saved;

      boost::optional<Gtk::TreeModel::RowReference> playing_row;
      boost::optional<Glib::ustring>                playing_uri;

      // External Callbacks
      void check_update (Glib::ustring const& location, Bmp::Library::TrackUpdated track);

    protected:

      virtual bool on_event (GdkEvent * ev);

    public:

      class Tracks : public Gtk::TreeModel::ColumnRecord
      {
        public:

          Gtk::TreeModelColumn<int>           track;
          Gtk::TreeModelColumn<Glib::ustring> title;
          Gtk::TreeModelColumn<int>           rating;
          Gtk::TreeModelColumn<Glib::ustring> artist;
          Gtk::TreeModelColumn<Glib::ustring> album;
          Gtk::TreeModelColumn<Glib::ustring> genre;
          Gtk::TreeModelColumn<int>           bitrate;
          Gtk::TreeModelColumn<int>           samplerate;
          Gtk::TreeModelColumn<Glib::ustring> location;
          Gtk::TreeModelColumn<bool>          present;
#ifdef HAVE_HAL
          Gtk::TreeModelColumn<Glib::ustring> volume_udi;
          Gtk::TreeModelColumn<Glib::ustring> vrp;
#endif

          Tracks ()
          {
            add (track);
            add (title);
            add (rating);
            add (artist);
            add (album);
            add (genre);
            add (bitrate);
            add (samplerate);
            add (location);
            add (present);
#ifdef HAVE_HAL
            add (volume_udi);
            add (vrp);
#endif
          }
      };
      Tracks tracks;

      void clear ();
      void unsort ();
      bool remove ();

      bool all_tracks_present ();

      void add_tracks (Bmp::DB::RowV const& vector,
                       Glib::ustring const& artist,
                       Glib::ustring const& album,
                       bool                 sortname);

      void get_files  (Util::FileList& list);

#ifdef HAVE_HAL
      void get_tracks (Bmp::Library::PUDIVRPV& pairs);
#else
      void get_tracks (Bmp::VFS::UriList& locations);
#endif

      void save_context  ()
      {
        store_tracks->get_sort_column_id (m_sort_column_saved, m_sort_order_saved);
      }

      void restore_context ()
      {
        store_tracks->clear();
        store_tracks->set_sort_column (m_sort_column_saved, m_sort_order_saved);
      }
  
      // PlaybackSource
      Glib::ustring playback_uri ();
      bool notify_play (unsigned int& position, unsigned int& size);
      bool notify_next (unsigned int& position, unsigned int& size);
      bool notify_prev (unsigned int& position, unsigned int& size);
      void notify_stop ();
      void notify_next_prev (bool &next, bool &prev);

      TrackView (BaseObjectType                        *cobject,
                 Glib::RefPtr<Gnome::Glade::Xml> const& xml);
      ~TrackView () {}
  };

  namespace UIPart
  {
    class Albums : public Bmp::PlaybackSource
    {
      private:

        typedef std::map < Glib::ustring, Gtk::TreeModel::iterator> IterMap;

#ifdef HAVE_HAL
        void hal_volume_removed  (Library::HAL::Volume volume);
        void hal_volume_added    (Library::HAL::Volume volume);
#endif

        void remove_tracks ();
        void import_tracks (Gtk::Widget *);
        void import_tracks_as_album (Gtk::Widget *);

        void approve_all ();
        void approve_selected ();
        void drop_items ();

        void remove_albums (bool force = false);
        void remove_albums_action_cb ();

        void relocate_album ();

        void albums_update ();
        void albums_modify ();
        void albums_set_sorting ();

        void tracks_selection_changed ();
        void albums_selection_changed ();

        void update_view_tracks ();

        void check_nextprev_caps ();
        void send_title ();

        void activate_default (Gtk::TreeModel::Path const& path, Gtk::TreeViewColumn* column);
        void mcs_view_size_changed (MCS_CB_DEFAULT_SIGNATURE);

        Glib::RefPtr<Gnome::Glade::Xml> m_ref_xml;

        Bmp::AlbumView        *view_albums;
        Bmp::TrackView        *view_tracks;

        Gtk::Entry            *albums_filter;
        Gtk::Notebook         *notebook_albums;
        Gtk::Notebook         *notebook_add;
        Gtk::ProgressBar      *progress_modify;
        Gtk::Label            *label_current_album;
        Gtk::ComboBox         *cb_view_size;

        boost::optional<Glib::ustring> m_saved_context_filter;

        enum ActionGroups { IMPORT, ALBUM, APPROVE, TRACKS, COLLECTIONS, N_ACTION_GROUPS };
        Glib::RefPtr<Gtk::UIManager>    m_ui_manager;
        Glib::RefPtr<Gtk::ActionGroup>  m_actions[N_ACTION_GROUPS];
        Glib::RefPtr<Gtk::ToggleAction> m_action_show_only_new;
        bool popup_menu (GdkEvent *ev);

        bool m_updated;
        bool m_busy;
        bool m_init;

        friend class Bmp::MainUI;

      protected:

        virtual Glib::ustring
        get_uri (); 

        virtual bool
        go_next ();

        virtual bool
        go_prev ();

        virtual void
        stop ();

        virtual void
        play ();

        virtual void
        play_requested () {}

        virtual void
        restore_context ();

        virtual GHashTable*
        get_metadata ();

      public:

        void import_external (VFS::UriList& urilist);

        Albums (Glib::RefPtr<Gnome::Glade::Xml> const& xml);
        ~Albums ();

    };
  }
}
#endif
